/*
  Copyright (c) 1998 Helsinki University of Technology.
  All rights reserved.

  Redistribution and use in source and binary forms are permitted
  provided that the above copyright notice and this paragraph are
  duplicated in all such forms and that any documentation,
  advertising materials, and other materials related to such
  distribution and use acknowledge that the software was developed
  by Helsinki University of Technology, Espoo.  The name of the
  University may not be used to endorse or promote products derived
  from this software without specific prior written permission.
  THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

/**
  Copyright (c) 1998 Pekka Nikander & Juha Parssinen
  All rights reserved.
  XXX   %Z%%M% version %I% date:%G%
  Module:  Adaptor
  Package: conduits
  File:    Adaptor.java
*/
package fi.hut.tcm.jacob.conduits;
/**
  Adaptor
  
  An adaptor is a Conduit that connects the conduit graph to the outside 
  world.  For example, a low level conduit graph could be connected to the 
  Ethernet driver through an Ethernet adaptor.
*/
public abstract class Adaptor
  extends Sedaptor
  implements AdaptorEventListener {

  /**
    Creates a new, empty adaptor. Used by subclasses.
  */
  protected Adaptor() {
  
    //+ Adaptor::Adaptor( )
    super();
    //- Adaptor::Adaptor( )
  }
  
  
  /**
  */
  protected final void accept(Visitor visitor, Side from)
    throws ConductingException, IllegalArgumentException  {
  
    //+ Adaptor::accept(Visitor, Side)
    visitor.at(this, from);
    //- Adaptor::accept(Visitor, Side)
  }
  
  
  /**
    Actually  handles an incoming message.
    Must be overridden by subclasses.
  */
  public abstract void adapt(Message message);

  /**
    Can be used to filter out certain messages.
    Returns true, if the message was consumed, and false otherwise.
    The default implementation filters out notification messages.
    Override if you want them checked.
    */
  protected boolean filter(Message message)
  {
    //+ Adaptor::filter(Message)
    // We might want to consider changing Adaptor.adapt() so that it would
    // by default just ignore notification messages, just forwarding
    // them as is. Currently it is an abstract method.
    // But I guess there is no sense in having an adaptor that
    // does nothing but checks if a message is a notification message
    // or not. (Terminators don't do even that much, though.)
    // Maybe we could call super, though, if we were not
    // interested in notification messages, but we might as well call
    // filter().
    Messenger messenger = message.getMessenger();
    if(messenger instanceof NotifyMessenger) {
      //System.out.println("Got notification, status " +
      //		 ((NotifyOob)message.getOob()).getStatusCode());
      return true;
    }
    return false;
    //- Adaptor::filter(Message)
  }
  
  
  /**
  */
  public int getSideCount() {
  
    //+ Adaptor::getSideCount( )
    return A + 1;
    //- Adaptor::getSideCount( )
  }
  
  
  /**
  */
    /* Cloning of Terminators fails because of this.
  public Object clone()
    throws CloneNotSupportedException  {
  
    //+ Adaptor::clone( )
    throw new CloneNotSupportedException("must override to support adaptor cloning");
    //- Adaptor::clone( )
  }
  */
  
} // Adaptor
