/*
  Copyright (c) 1998 Helsinki University of Technology.
  All rights reserved.

  Redistribution and use in source and binary forms are permitted
  provided that the above copyright notice and this paragraph are
  duplicated in all such forms and that any documentation,
  advertising materials, and other materials related to such
  distribution and use acknowledge that the software was developed
  by Helsinki University of Technology, Espoo.  The name of the
  University may not be used to endorse or promote products derived
  from this software without specific prior written permission.
  THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

/**
  XXX   %Z%%M% version %I% date:%G%
  Component:  BufferMessenger
  Package: conduits
  File:    BufferMessenger.java
*/

package fi.hut.tcm.jacob.conduits;

/**
  BufferMessenger
  
  
*/
public class BufferMessenger
  extends Object {

  /**
  */
  public static Object newBuffer(int maxsize, int initialoffset, int initiallength) {
  
  //+ BufferMessenger::newBuffer(int, int, int)
    BufferData bufdata = new BufferMessenger.BufferData();
    bufdata.data = new byte[maxsize];
    bufdata.offset = initialoffset;
    bufdata.length = initiallength;
    return bufdata;
  //- BufferMessenger::newBuffer(int, int, int)
  }
  
  /**
  */
  public static Object newBuffer(int maxsize, int initialoffset, byte[] initialdata) {
  
  //+ BufferMessenger::newBuffer(int, int, byte[])
    BufferData bufdata = new BufferMessenger.BufferData();
    bufdata.data = new byte[maxsize];
    bufdata.offset = initialoffset;
    bufdata.length = initialdata.length;
    System.arraycopy(initialdata, 0, bufdata.data, initialoffset,
		     initialdata.length);
    return bufdata;
  //- BufferMessenger::newBuffer(int, int, byte[])
  }
  
  /**
  */
  public static Object newBuffer(int maxsize, int initialoffset, byte[] initialdata, int initiallength) {
  
  //+ BufferMessenger::newBuffer(int, int, byte[], int)
    BufferData bufdata = new BufferMessenger.BufferData();
    bufdata.data = new byte[maxsize];
    bufdata.offset = initialoffset;
    bufdata.length = initiallength;
    System.arraycopy(initialdata, 0, bufdata.data, initialoffset, initiallength);
    return bufdata;
  //- BufferMessenger::newBuffer(int, int, byte[], int)
  }
  
  /**
  */
  public byte[] extractData(Object bufRef) {
  
  //+ BufferMessenger::extractData(Object)
    BufferData bufdata = (BufferData)bufRef;
    byte[] data = new byte[bufdata.length];
    System.arraycopy(bufdata.data, bufdata.offset, data, 0, bufdata.length);
    return data;
  //- BufferMessenger::extractData(Object)
  }
  
  /**
  */
  public int getLength(Object bufRef) {
  
  //+ BufferMessenger::getLength(Object)
    return ((BufferData)bufRef).length;
  //- BufferMessenger::getLength(Object)
  }
  
  /**
  */
  public int getByte(Object bufRef, int index) {
  
  //+ BufferMessenger::getByte(Object, int)
    BufferData bufdata = (BufferData)bufRef;
    return bufdata.data[index + bufdata.offset];
  //- BufferMessenger::getByte(Object, int)
  }
  
  /**
  */
  public void setByte(Object bufRef, int index, int value) {
  
  //+ BufferMessenger::setByte(Object, int, int)
    BufferData bufdata = (BufferData)bufRef;
    bufdata.data[index + bufdata.offset] = (byte) (value & 0xff);
  //- BufferMessenger::setByte(Object, int, int)
  }
  
  /**
  */
  public int getHalfByte(Object bufRef, int hboff) {
  
  //+ BufferMessenger::getHalfByte(Object, int)
    BufferData bufdata =  (BufferData)bufRef;
    int b = (int)bufdata.data[bufdata.offset + (hboff>>1)];
    return (((hboff%2)==0)?(b>>4):(b&0xf));
  //- BufferMessenger::getHalfByte(Object, int)
  }
  
  /**
  */
  public void setHalfByte(Object bufRef, int hboff, int value) {
  
  //+ BufferMessenger::setHalfByte(Object, int, int)
    BufferData bufdata =  (BufferData)bufRef;
    int b = (int)bufdata.data[bufdata.offset + (hboff>>1)];
    b = (((hboff%2)==0)?
	 ((b&0xf0)|(value&0xf)):
	 (((value&0xf)<<4)|(b&0xf)));
    bufdata.data[bufdata.offset + (hboff>>1)] = (byte)b;
  //- BufferMessenger::setHalfByte(Object, int, int)
  }
  
  /**
  */
  public int getBit(Object bufRef, int bitoff) {
  
  //+ BufferMessenger::getBit(Object, int)
    BufferData bufdata = (BufferData)bufRef;
    int b = (int)bufdata.data[bufdata.offset + (bitoff>>4)];
    int bbitoff = 7 - (bitoff % 8);
    return (b>>bbitoff)&0x1;
  //- BufferMessenger::getBit(Object, int)
  }
  
  /**
  */
  public void setBit(Object bufRef, int bitoff, int value) {
  
  //+ BufferMessenger::setBit(Object, int, int)
    BufferData bufdata = (BufferData)bufRef;
    int bbitoff = 7 - (bitoff % 8);
    value &= 0x1;
    if(value==0)
      bufdata.data[bufdata.offset + (bitoff>>4)] &= ~(1 << bbitoff);
    else
      bufdata.data[bufdata.offset + (bitoff>>4)] |= (1 << bbitoff);
  //- BufferMessenger::setBit(Object, int, int)
  }
  
  /**
  */
  public int getShort(Object bufRef, int index) {
  
  //+ BufferMessenger::getShort(Object, int)
    return getShortUnaligned(bufRef, 2 * index);
  //- BufferMessenger::getShort(Object, int)
  }
  
  /**
  */
  public int getShortUnaligned(Object bufRef, int byteoffset) {
  
  //+ BufferMessenger::getShortUnaligned(Object, int)
    BufferData bufdata = (BufferData)bufRef;

    return ((( (int) bufdata.data[bufdata.offset + byteoffset + 0]) &
	     0xff) << 8)
         | ((( (int) bufdata.data[bufdata.offset + byteoffset + 1]) &
	     0xff) << 0);
  //- BufferMessenger::getShortUnaligned(Object, int)
  }
  
  /**
  */
  public void setShort(Object bufRef, int index, int value) {
  
  //+ BufferMessenger::setShort(Object, int, int)
    setShortUnaligned(bufRef, index * 2, value);
  //- BufferMessenger::setShort(Object, int, int)
  }
  
  /**
  */
  public void setShortUnaligned(Object bufRef, int byteoffset, int value) {
  
  //+ BufferMessenger::setShortUnaligned(Object, int, int)
    BufferData bufdata = (BufferData)bufRef;
    bufdata.data[bufdata.offset + byteoffset + 0] = (byte)(value >> 8);
    bufdata.data[bufdata.offset + byteoffset + 1] = (byte)(value >> 0);
  //- BufferMessenger::setShortUnaligned(Object, int, int)
  }
  
  /**
  */
  public int getInt(Object bufRef, int index) {
  
  //+ BufferMessenger::getInt(Object, int)
    return getIntUnaligned(bufRef, index * 4);
  //- BufferMessenger::getInt(Object, int)
  }
  
  /**
  */
  public int getIntUnaligned(Object bufRef, int byteoffset) {
  
  //+ BufferMessenger::getIntUnaligned(Object, int)
    BufferData bufdata =  (BufferData)bufRef;
    return ((((int) bufdata.data[bufdata.offset + byteoffset + 0]) & 0xff) << 24)
    	 | ((((int) bufdata.data[bufdata.offset + byteoffset + 1]) & 0xff) << 16)
    	 | ((((int) bufdata.data[bufdata.offset + byteoffset + 2]) & 0xff) <<  8)
    	 | ((((int) bufdata.data[bufdata.offset + byteoffset + 3]) & 0xff) <<  0);
  //- BufferMessenger::getIntUnaligned(Object, int)
  }
  
  /**
  */
  public void setInt(Object bufRef, int index, int value) {
  
  //+ BufferMessenger::setInt(Object, int, int)
    setIntUnaligned(bufRef, index * 4, value);
  //- BufferMessenger::setInt(Object, int, int)
  }
  
  /**
  */
  public void setIntUnaligned(Object bufRef, int byteoffset, int value) {
  
  //+ BufferMessenger::setIntUnaligned(Object, int, int)
    BufferData bufdata =  (BufferData)bufRef;
    bufdata.data[bufdata.offset + byteoffset + 0] = (byte)(value >> 24);
    bufdata.data[bufdata.offset + byteoffset + 1] = (byte)(value >> 16);
    bufdata.data[bufdata.offset + byteoffset + 2] = (byte)(value >>  8);
    bufdata.data[bufdata.offset + byteoffset + 3] = (byte)(value >>  0);
  //- BufferMessenger::setIntUnaligned(Object, int, int)
  }
  
  /**
  */
  public long get6BytesUnaligned(Object bufRef, int byteoffset) {
  
  //+ BufferMessenger::get6BytesUnaligned(Object, int)
    BufferData bufdata =  (BufferData)bufRef;
    return ((((long) bufdata.data[bufdata.offset + byteoffset + 0]) & 0xff) << 40)
    	 | ((((long) bufdata.data[bufdata.offset + byteoffset + 1]) & 0xff) << 32)
    	 | ((((long) bufdata.data[bufdata.offset + byteoffset + 2]) & 0xff) << 24)
    	 | ((((long) bufdata.data[bufdata.offset + byteoffset + 3]) & 0xff) << 16)
         | ((((long) bufdata.data[bufdata.offset + byteoffset + 4]) & 0xff) <<  8)
    	 | ((((long) bufdata.data[bufdata.offset + byteoffset + 5]) & 0xff) <<  0);
  //- BufferMessenger::get6BytesUnaligned(Object, int)
  }
  
  /**
  */
  public void set6BytesUnaligned(Object bufRef, int byteoffset, long value) {
  
  //+ BufferMessenger::set6BytesUnaligned(Object, int, long)
    BufferData bufdata =  (BufferData)bufRef;
    bufdata.data[bufdata.offset + byteoffset + 0] = (byte)(value >> 40);
    bufdata.data[bufdata.offset + byteoffset + 1] = (byte)(value >> 32);
    bufdata.data[bufdata.offset + byteoffset + 2] = (byte)(value >> 24);
    bufdata.data[bufdata.offset + byteoffset + 3] = (byte)(value >> 16);
    bufdata.data[bufdata.offset + byteoffset + 4] = (byte)(value >>  8);
    bufdata.data[bufdata.offset + byteoffset + 5] = (byte)(value >>  0);
  //- BufferMessenger::set6BytesUnaligned(Object, int, long)
  }
  
  /**
  */
  public long getLong(Object bufRef, int index) {
  
  //+ BufferMessenger::getLong(Object, int)
    return getLongUnaligned(bufRef, index * 8);
  //- BufferMessenger::getLong(Object, int)
  }
  
  /**
  */
  public long getLongUnaligned(Object bufRef, int byteoffset) {
  
  //+ BufferMessenger::getLongUnaligned(Object, int)
    BufferData bufdata =  (BufferData)bufRef;
    return ((((int) bufdata.data[bufdata.offset + byteoffset + 0]) & 0xff) << 56)
    	 | ((((int) bufdata.data[bufdata.offset + byteoffset + 1]) & 0xff) << 48)
    	 | ((((int) bufdata.data[bufdata.offset + byteoffset + 2]) & 0xff) << 40)
    	 | ((((int) bufdata.data[bufdata.offset + byteoffset + 3]) & 0xff) << 32)
         | ((((int) bufdata.data[bufdata.offset + byteoffset + 4]) & 0xff) << 24)
    	 | ((((int) bufdata.data[bufdata.offset + byteoffset + 5]) & 0xff) << 16)
    	 | ((((int) bufdata.data[bufdata.offset + byteoffset + 6]) & 0xff) <<  8)
    	 | ((((int) bufdata.data[bufdata.offset + byteoffset + 7]) & 0xff) <<  0);
  //- BufferMessenger::getLongUnaligned(Object, int)
  }
  
  /**
  */
  public void setLong(Object bufRef, int index, long value) {
  
  //+ BufferMessenger::setLong(Object, int, long)
    setLongUnaligned(bufRef, index * 8, value);
  //- BufferMessenger::setLong(Object, int, long)
  }
  
  /**
  */
  public void setLongUnaligned(Object bufRef, int byteoffset, long value) {
  
  //+ BufferMessenger::setLongUnaligned(Object, int, long)
    BufferData bufdata =  (BufferData)bufRef;
    bufdata.data[bufdata.offset + byteoffset + 0] = (byte)(value >> 56);
    bufdata.data[bufdata.offset + byteoffset + 1] = (byte)(value >> 48);
    bufdata.data[bufdata.offset + byteoffset + 2] = (byte)(value >> 40);
    bufdata.data[bufdata.offset + byteoffset + 3] = (byte)(value >> 32);
    bufdata.data[bufdata.offset + byteoffset + 4] = (byte)(value >> 24);
    bufdata.data[bufdata.offset + byteoffset + 5] = (byte)(value >> 16);
    bufdata.data[bufdata.offset + byteoffset + 6] = (byte)(value >>  8);
    bufdata.data[bufdata.offset + byteoffset + 7] = (byte)(value >>  0);
  //- BufferMessenger::setLongUnaligned(Object, int, long)
  }
  
  /**
  */
  public void hexDump(Object bufRef) {
  
  //+ BufferMessenger::hexDump(Object)
    hexDump(bufRef, 0, getLength(bufRef), 16);
  //- BufferMessenger::hexDump(Object)
  }
  
  /**
  */
  public void hexDump(Object bufRef, int offset, int size, int alignment) {
  
  //+ BufferMessenger::hexDump(Object, int, int, int)
    BufferData bufdata =  (BufferData)bufRef;
    if (offset < 0 || offset + size > bufdata.length) {
        throw new ArrayIndexOutOfBoundsException("XXX");
      }
      int beg = offset + bufdata.offset;

      int i = 0; 
      while (i < size) {
        if (((int) bufdata.data[beg + i] & 0xff) > 0xf) {
          System.err.print(Integer.toHexString((int) bufdata.data[beg + i] & 0xff));
        } else {
          System.err.print("0" + 
                           Integer.toHexString((int) bufdata.data[beg + i] & 0xff));
        }
        i++;
        if ((alignment != 0 && i % alignment == 0) || i == size) {
          System.err.println();
        } else {
          System.err.print(" ");
        }
      }
  //- BufferMessenger::hexDump(Object, int, int, int)
  }
  
  /**
  */
  public boolean prependHeader(Object bufRef, int numBytes) {
  
  //+ BufferMessenger::prependHeader(Object, int)
    BufferData bufdata =  (BufferData)bufRef;
    if (bufdata.offset < numBytes)
      return false;  // fail, perhaps we should use exceptions instead?
    bufdata.offset -= numBytes;
    bufdata.length += numBytes;
    return true;
  //- BufferMessenger::prependHeader(Object, int)
  }
  
  /**
  */
  public void removeHeader(Object bufRef, int numBytes) {
  
  //+ BufferMessenger::removeHeader(Object, int)
    BufferData bufdata =  (BufferData)bufRef;
    // XXX cms: we should throw an exception in case
    // of illegal use, for instance Array..outOfBoundsException.
    bufdata.offset += numBytes;
    bufdata.length -= numBytes;
  //- BufferMessenger::removeHeader(Object, int)
  }
  
  /**
  */
  public boolean appendTrailer(Object bufRef, int numBytes) {
  
  //+ BufferMessenger::appendTrailer(Object, int)
    BufferData bufdata =  (BufferData)bufRef;
    if (bufdata.offset + bufdata.length + numBytes > bufdata.data.length)
      return false;  // XXX fail, perhaps we should use exceptions instead?
    bufdata.length += numBytes;
    return true;
  //- BufferMessenger::appendTrailer(Object, int)
  }
  
  /**
  */
  public void removeTrailer(Object bufRef, int numBytes) {
  
  //+ BufferMessenger::removeTrailer(Object, int)
    BufferData bufdata =  (BufferData)bufRef;
    bufdata.length -= numBytes;
  //- BufferMessenger::removeTrailer(Object, int)
  }
  
  /**
  */
  public void copyBuffer(Object src, int soffs, Object dst, int doffs, int size) {
  
  //+ BufferMessenger::copyBuffer(Object, int, Object, int, int)
    if (size + soffs > getLength(src) || size + doffs > getLength(dst)) {
      throw new IndexOutOfBoundsException("src: " + soffs + " + " + size +
                                          "  dst: " + doffs + " + " + size);
    }
    soffs += ((BufferData)src).offset;
    doffs += ((BufferData)dst).offset;
    System.arraycopy(((BufferData)src).data, soffs,
                     ((BufferData)dst).data, doffs,
                     size);
  //- BufferMessenger::copyBuffer(Object, int, Object, int, int)
  }
  
  /**
  */
  public void copyData(byte[] src, int soffs, Object dst, int doffs, int size) {
  
  //+ BufferMessenger::copyData(byte[], int, Object, int, int)
    if (size + soffs > src.length || size + doffs > getLength(dst)) {
      throw new IndexOutOfBoundsException("src: " + soffs + " + " + size +
                                          "  dst: " + doffs + " + " + size);
    }
    doffs += ((BufferData)dst).offset;
    System.arraycopy(src, soffs, ((BufferData)dst).data, doffs, size);
  //- BufferMessenger::copyData(byte[], int, Object, int, int)
  }
  
  /**
    BufferData (inner class)
    
    A C-struct-like class that defines the data objects created
    and manipulated by BufferMessenger.
  */
  static class BufferData
    extends Object {
  
    /**
    */
    public int offset;
    
    /**
    */
    public int length;
    
    /**
    */
    public byte[] data;
    
  } // BufferData
  
} // BufferMessenger

// +*************** UNUSED CODE ***************

// -*************** UNUSED CODE ***************
