/*
  Copyright (c) 1998 Helsinki University of Technology.
  All rights reserved.

  Redistribution and use in source and binary forms are permitted
  provided that the above copyright notice and this paragraph are
  duplicated in all such forms and that any documentation,
  advertising materials, and other materials related to such
  distribution and use acknowledge that the software was developed
  by Helsinki University of Technology, Espoo.  The name of the
  University may not be used to endorse or promote products derived
  from this software without specific prior written permission.
  THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

/**
  Copyright (C) 1998 XYZ Corporation. All rights Reserved.
  XXX   %Z%%M% version %I% date:%G%
  Component:  InstallMessenger
  Package: conduits
  File:    InstallMessenger.java
*/

package fi.hut.tcm.jacob.conduits;

/**
  InstallMessenger
  
  Carries Installation information for ConduitFactories.
  When a Message arrives at a ConduitFactory, the factory
  determines if the Message's Messenger is an InstallMessenger.  If so, the
   ConduitFactory queries the message about the exact operation to perform.
   The InstallMessenger is used to perform the query. If not so, the 
  ConduitFactory uses its internal resources.
*/
public interface InstallMessenger
  extends Messenger {

  /**
    Returns a Side (corresponding to <code>index</index>) where the cloned 
    Conduit is to be attached to, or null if the Side is to be determined 
    by the ConduitFactory itself.
    
    @param message message where to look for the Sides
    @param index typically Side.A or Side.B, i.e., index of the side that 
    is to be attached to.
    @return a Side to attach, or null.
  */
  public abstract Side getEndpoint(Message message, int index);
  
  
  /**
  */
  public abstract int getEndpointCount(Message message);
  
  
  /**
    Returns a prototype conduit that the ConduitFactory attempts to clone, 
    or null if to use the prototype stored at the factory.
    @param message the message that carries the conduit
    @return a Conduit to clone, or null
    
  */
  public abstract Conduit getPrototype(Message message);
  
  
  /**
  */
  public abstract Object getKey(Message message, int index);
  
  
} // InstallMessenger

// +*************** UNUSED CODE ***************
// -*************** UNUSED CODE ***************
