/*
  Copyright (c) 1998 Helsinki University of Technology.
  All rights reserved.

  Redistribution and use in source and binary forms are permitted
  provided that the above copyright notice and this paragraph are
  duplicated in all such forms and that any documentation,
  advertising materials, and other materials related to such
  distribution and use acknowledge that the software was developed
  by Helsinki University of Technology, Espoo.  The name of the
  University may not be used to endorse or promote products derived
  from this software without specific prior written permission.
  THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

/**
  Copyright (C) 1998 XYZ Corporation. All rights Reserved.
  XXX   %Z%%M% version %I% date:%G%
  Component:  InstallOob
  Package: conduits
  File:    InstallOob.java
*/

package fi.hut.tcm.jacob.conduits;

/**
  InstallOob
  
  
*/
public class InstallOob
  extends Object
  implements OutOfBand {

  /**
  */
  private Side[] endpoints;
  
  /**
         If a Side is a MuxSide, a key will be needed to know
         which MuxingSide to use or create.
    
  */
  private Object[] keys;
  
  /**
  */
  private Conduit prototype;
  
  /* ****************** Oob ************************/
  
  /**
    A Message may carry OutOfBand data. This association
    allows one to access the OOB information
  */

  /* ***************End of Oob *********************/
  
  /**
  */
  public InstallOob(Side[] endpoints, Conduit proto) {
  
  //+ InstallOob::InstallOob(Side[], Conduit)
    if (endpoints == null)
      throw new IllegalArgumentException("endpoints can't be null;"+
					 " empty array is ok");
    this.endpoints = endpoints;
    this.prototype = proto;
    this.keys = new Object[endpoints.length];
    for(int i=0; i<endpoints.length; i++)
      this.keys[i] = null;
  //- InstallOob::InstallOob(Side[], Conduit)
  }
  
  /**
  */
  public InstallOob(Side[] endpoints, Object[] keys, Conduit proto) {
  
  //+ InstallOob::InstallOob(Side[], Object[], Conduit)
    if (endpoints == null)
      throw new IllegalArgumentException("endpoints can't be null;"+
					 " empty array is ok");
    this.endpoints = endpoints;
    this.prototype = proto;
    this.keys = keys;
  //- InstallOob::InstallOob(Side[], Object[], Conduit)
  }
  
  /**
  */
  public int getEndpointCount() {
  
  //+ InstallOob::getEndpointCount( )
    return endpoints.length;
  //- InstallOob::getEndpointCount( )
  }
  
  /**
  */
  public Side getEndpoint(int index) {
  
  //+ InstallOob::getEndpoint(int)
    return endpoints[index];
  //- InstallOob::getEndpoint(int)
  }
  
  /**
  */
  public void setEndpointCount(int count) {
  
  //+ InstallOob::setEndpointCount(int)
    Side[] newEndpoints = new Side[count];
    System.arraycopy(endpoints, 0, newEndpoints, 0, 
                     endpoints.length > count? count: endpoints.length);
    endpoints = newEndpoints;

    Object[] newKeys = new Object[count];
    System.arraycopy(keys, 0, newKeys, 0, 
                     keys.length > count ? count : keys.length);
    keys = newKeys;
  //- InstallOob::setEndpointCount(int)
  }
  
  /**
  */
  public void setEndpoint(int index, Side endpoint) {
  
  //+ InstallOob::setEndpoint(int, Side)
    endpoints[index] = endpoint;
  //- InstallOob::setEndpoint(int, Side)
  }
  
  /**
  */
  public Conduit getPrototype() {
  
  //+ InstallOob::getPrototype( )
    return prototype;
  //- InstallOob::getPrototype( )
  }
  
  /**
  */
  public void setPrototype(Conduit protot) {
  
  //+ InstallOob::setPrototype(Conduit)
    prototype = protot;
  //- InstallOob::setPrototype(Conduit)
  }
  
  /**
         The EndpointCount methods also affect the key count.
  */
  public Object getKey(int index) {
  
  //+ InstallOob::getKey(int)
      return keys[index];
  //- InstallOob::getKey(int)
  }
  
  /**
         The EndpointCount methods also affect the key count.
  */
  public void setKey(int index, Object key) {
  
  //+ InstallOob::setKey(int, Object)
      keys[index] = key;
  //- InstallOob::setKey(int, Object)
  }
  
} // InstallOob

// +*************** UNUSED CODE ***************
// -*************** UNUSED CODE ***************
