/*
  Copyright (c) 1998 Helsinki University of Technology.
  All rights reserved.

  Redistribution and use in source and binary forms are permitted
  provided that the above copyright notice and this paragraph are
  duplicated in all such forms and that any documentation,
  advertising materials, and other materials related to such
  distribution and use acknowledge that the software was developed
  by Helsinki University of Technology, Espoo.  The name of the
  University may not be used to endorse or promote products derived
  from this software without specific prior written permission.
  THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

/**
  Copyright (c) 1998 Pekka Nikander & Juha Parssinen
  All rights reserved.
  XXX   %Z%%M% version %I% date:%G%
  Module:  Notifier
  Package: conduits
  File:    Notifier.java
*/
package fi.hut.tcm.jacob.conduits;
/**
  Notifier
  
  This class implements the abstract class NotifyMessenger.
  If you use an Notifier object as a messenger, you must
  use NotifyOob as the out of band object of the message.
*/
public final class Notifier
  extends Object
  implements NotifyMessenger, Messenger {

  /**
     Notifier is a singleton, get a reference from here.
  */
  public static final Notifier instance = new Notifier();
  
  /**
  */
  private Notifier()
    {
  
      //+ Notifier::Notifier( )
      //- Notifier::Notifier( )
    }
  
  /**
   */
  public Message getRequest(Message message)
    {
      //+ Notifier::getRequest(Message)
      NotifyOob oob = (NotifyOob)message.getOob();
      return oob.getMessage();
      //- Notifier::getRequest(Message)
    }
  
  
  /**
  */
  public int getStatusCode(Message message)
    {
      //+ Notifier::getStatusCode(Message)
      NotifyOob oob = (NotifyOob)message.getOob();
      return oob.getStatusCode();
      //- Notifier::getStatusCode(Message)
    }

  /**
  */
  public void apply(Message message, State state, Session session)
    {
      
      //+ Notifier::apply(Message, State, Session)
      // Do nothing
      //- Notifier::apply(Message, State, Session)
    }
  
  
} // Notifier


// +*************** UNUSED CODE ***************
// -*************** UNUSED CODE ***************
