/*
  Copyright (c) 1998 Helsinki University of Technology.
  All rights reserved.

  Redistribution and use in source and binary forms are permitted
  provided that the above copyright notice and this paragraph are
  duplicated in all such forms and that any documentation,
  advertising materials, and other materials related to such
  distribution and use acknowledge that the software was developed
  by Helsinki University of Technology, Espoo.  The name of the
  University may not be used to endorse or promote products derived
  from this software without specific prior written permission.
  THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

/**
  Copyright (c) 1998 Pekka Nikander & Juha Parssinen
  All rights reserved.
  XXX   %Z%%M% version %I% date:%G%
  Module:  Sedaptor
  Package: conduits
  File:    Sedaptor.java
*/
package fi.hut.tcm.jacob.conduits;
/**
  Sedaptor
  
  Common hidden superclass to Session and Adaptor
*/
public abstract class Sedaptor
  extends BaseConduit {

  /**
  */
  protected Sedaptor() {
  
    //+ Sedaptor::Sedaptor( )
    //- Sedaptor::Sedaptor( )
  }
  
  
  /**
  */
  protected final Object addTimeout(Side sentTo, long timeout, Message message) {
  
    //+ Sedaptor::addTimeout(Side, long, Message)
    return Timeout.newTimeout(sentTo, message, timeout);
    //- Sedaptor::addTimeout(Side, long, Message)
  }
  
  
  /**
  */
  protected final void cancelTimeout(Object handle) {
  
    //+ Sedaptor::cancelTimeout(Object)
    ((Timeout)handle).cancel();
    //- Sedaptor::cancelTimeout(Object)
  }
  
  
  /**
  */
  protected Message newMessage(Object contents, Messenger messenger, OutOfBand oob) {
  
    //+ Sedaptor::newMessage(Object, Messenger, OutOfBand)
    return Message.newMessage(contents, messenger, oob);
    //- Sedaptor::newMessage(Object, Messenger, OutOfBand)
  }
  
  
  /**
  */
  protected void sendMessage(Message m, Side source) {
  
    //+ Sedaptor::sendMessage(Message, Side)
    for (int i = 0; i < getSideCount(); i++) {
      if (sides[i] == source)
        break;
    }
    throw new IllegalArgumentException("XXX");
    // XXX
    //- Sedaptor::sendMessage(Message, Side)
  }
  
  
  /**
  */
  protected void returnMessage(Message m) {
  
    //+ Sedaptor::returnMessage(Message)
    Side source = m.getSourceSide();
    if (!(source instanceof BaseSide)) 
       throw new IllegalStateException("XXX");
    m.setSourceSide(((BaseSide)source).getOpposite());
    //- Sedaptor::returnMessage(Message)
  }
  
  
  /**
  */
  public void stopMessage(Message m) {
  
    //+ Sedaptor::stopMessage(Message)
    m.setSourceSide(null);
    //- Sedaptor::stopMessage(Message)
  }
  
  
  /**
  */
  public void releaseMessage(Message m) {
  
    //+ Sedaptor::releaseMessage(Message)
    stopMessage(m);
    m.setTransporter(null);
    m.finalize();
    //- Sedaptor::releaseMessage(Message)
  }
  
  
  /**
  */
  protected Side getMessageSource(Message m) {
  
    //+ Sedaptor::getMessageSource(Message)
    return m.getSourceSide();
    //- Sedaptor::getMessageSource(Message)
  }
  
  
} // Sedaptor
