/*
  Copyright (c) 1998 Helsinki University of Technology.
  All rights reserved.

  Redistribution and use in source and binary forms are permitted
  provided that the above copyright notice and this paragraph are
  duplicated in all such forms and that any documentation,
  advertising materials, and other materials related to such
  distribution and use acknowledge that the software was developed
  by Helsinki University of Technology, Espoo.  The name of the
  University may not be used to endorse or promote products derived
  from this software without specific prior written permission.
  THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

/**
  Copyright (c) 1998 Pekka Nikander & Juha Parssinen
  All rights reserved.
  XXX   %Z%%M% version %I% date:%G%
  Module:  Session
  Package: conduits
  File:    Session.java
*/
package fi.hut.tcm.jacob.conduits;
/**
  Session
  
  A session represents an (active) protocol session or connection.
  Connectionless protocols (e.g. IP or UDP) typically contain one session 
  per protocol entity (e.g. IP address or UDP port). Connection-oriented 
  protocols (e.g. TCP), on the other hand, typically contain a session for 
  each active connection.
*/
public abstract class Session
  extends Sedaptor
  implements SessionEventListener {

  /**
  */
  private State state;
  
  /**
  */
  protected Session(State initialState) {
  
    //+ Session::Session(State)
    state = initialState;
    //- Session::Session(State)
  }
  
  
  /**
  */
  protected final void accept(Visitor visitor, Side from)
    throws ConductingException, IllegalArgumentException  {
  
    //+ Session::accept(Visitor, Side)
      visitor.at(this, from);  
    //- Session::accept(Visitor, Side)
  }
  
  
  /**
  */
  public void apply(Message message) {
  
    //+ Session::apply(Message)
    // There must always be a messenger
    if(Debugger.level > 3)
      System.out.println("Session.apply()");
    message.getMessenger().apply(message, state, this);
  //- Session::apply(Message)
  }
  
  
  /**
  */
  public void setState(State state)
    throws IllegalArgumentException  {
  
    //+ Session::setState(State)
    if (state == null) 
      throw new IllegalArgumentException("null state");
    this.state = state;
    //- Session::setState(State)
  }
  
  
} // Session


