/*
  Copyright (c) 1998 Helsinki University of Technology.
  All rights reserved.

  Redistribution and use in source and binary forms are permitted
  provided that the above copyright notice and this paragraph are
  duplicated in all such forms and that any documentation,
  advertising materials, and other materials related to such
  distribution and use acknowledge that the software was developed
  by Helsinki University of Technology, Espoo.  The name of the
  University may not be used to endorse or promote products derived
  from this software without specific prior written permission.
  THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

/**
  Copyright (c) 1998 Pekka Nikander & Juha Parssinen
  All rights reserved.
  XXX   %Z%%M% version %I% date:%G%
  Module:  Pool
  Package: utility
  File:    Pool.java
*/
package fi.hut.tcm.jacob.utility;
/**
  Pool
  
  A Pool is used to store objects i.e. MessageTransporters, Controllers or 
  Timeouts (i.e. Visitors).
*/
public final class Pool
  extends Buffer
  implements Storage {

  /**
    This is overloaded constructor of the Pool. It uses defaultsize for new
     storage
  */
  public Pool() {
  
    //+ Pool::Pool( )
    super();
    //- Pool::Pool( )
  }
  
  
  /**
    This is overloaded constructor of the Queue. It uses given initialSize 
    for new storage
  */
  public Pool(int initialSize) {
  
    //+ Pool::Pool(int)
    super(initialSize);
    //- Pool::Pool(int)
  }
  
  
  /**
    Returns true if storage is full otherwise false. This method is used 
    unsynchronized. 
  */
  public boolean full() {
  
    //+ Pool::full( )
    return super.full();
    //- Pool::full( )
  }
  
  
  /**
    Return true if storage is empty otherwise false.  This method is used 
    unsynchronized
  */
  public boolean empty() {
  
    //+ Pool::empty( )
    return super.empty();
    //- Pool::empty( )
  }
  
  
  /**
    Adds item to the Pool. This is used synchronized.
  */
  public void put(Object item) {
  
    //+ Pool::put(Object)
    if (full()) 
      enlarge();
    putTail(item);
    //- Pool::put(Object)
  }
  
  
  /**
    Removes one Item from Pool and returns reference to it. If the Queue is
     empty returns null. This is used synchronized.. 
  */
  public Object take() {
  
    //+ Pool::take( )
    if (empty()) 
      return null;
    return takeHead();
    //- Pool::take( )
  }
  
  
} // Pool

