/*
  Copyright (c) 1998 Helsinki University of Technology.
  All rights reserved.

  Redistribution and use in source and binary forms are permitted
  provided that the above copyright notice and this paragraph are
  duplicated in all such forms and that any documentation,
  advertising materials, and other materials related to such
  distribution and use acknowledge that the software was developed
  by Helsinki University of Technology, Espoo.  The name of the
  University may not be used to endorse or promote products derived
  from this software without specific prior written permission.
  THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

/**
  Copyright (c) 1998 Pekka Nikander & Juha Parssinen
  All rights reserved.
  XXX   %Z%%M% version %I% date:%G%
  Module:  Queue
  Package: utility
  File:    Queue.java
*/
package fi.hut.tcm.jacob.utility;
/**
  Queue
  
  A Queue is used to store Items which can be MessageTransporters, or 
  Controllers (i.e. Visitors).
*/
public class Queue
  extends Buffer
  implements Storage {

  /**
    This is overloaded constructor of the Queue. It uses defaultsize for 
    new storage.
  */
  public Queue() {
  
    //+ Queue::Queue( )
    super();
    //- Queue::Queue( )
  }
  
  
  /**
    This is overloaded constructor of the Queue. It uses given initialSize 
    for new storage.
  */
  public Queue(int initialSize) {
  
    //+ Queue::Queue(int)
    super(initialSize);
    //- Queue::Queue(int)
  }
  
  
  /**
    Returns true if storage is full otherwise false. This method is used 
    unsynchronized. 
  */
  public boolean full() {
  
    //+ Queue::full( )
    return super.full();
    //- Queue::full( )
  }
  
  
  /**
    Return true if storage is empty otherwise false.  This method is used 
    unsynchronized. 
  */
  public boolean empty() {
  
    //+ Queue::empty( )
    return super.empty();
    //- Queue::empty( )
  }
  
  
  /**
    Adds item to storage and notifyes all waiting take method calls. This 
    is used synchronized.
  */
  public synchronized void put(Object item) {
  
    //+ Queue::put(Object)
    if (full()) 
      enlarge();
    putTail(item);
    notifyAll();
    //- Queue::put(Object)
  }
  
  
  /**
    Removes one Item from Queue and returns reference to it. If the Queue 
    is empty this method call blocks until notifyed (by put method). This 
    is used synchronized.
  */
  public synchronized Object take() {
  
    //+ Queue::take( )
    while  (empty()) try { 
      wait();
    } catch(InterruptedException ex) {
    }
    return takeHead();
    //- Queue::take( )
  }
  
  
} // Queue

